/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdio.h>
#include <errno.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/sys/byteorder.h>
#include <zephyr/sys/printk.h>
#include <zephyr/logging/log.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/spi.h>
#include <nrfx_gpiote.h>


#include <zephyr/irq.h>

#define LOG_MODULE_NAME app
LOG_MODULE_REGISTER(LOG_MODULE_NAME);

#define SPIOP       SPI_WORD_SET(8) | SPI_TRANSFER_MSB

struct spi_dt_spec spispec = SPI_DT_SPEC_GET(DT_NODELABEL(nrf_radio_fem_spi), SPIOP, 0);

#define NRF_FEM_GPIO ((NRF_GPIO_Type *)DT_REG_ADDR(DT_GPIO_CTLR(DT_PHANDLE(DT_NODELABEL(radio), fem), pdn_gpios)))
#define NRF_GPIO_PDN_PIN    DT_GPIO_PIN(DT_PHANDLE(DT_NODELABEL(radio), fem), pdn_gpios)


void test_spi(void)
{
    int err;

    err = spi_is_ready_dt(&spispec);
    LOG_INF("spi_is_ready_dt() returned 0x%0x", err);

    NRF_FEM_GPIO->DIRSET = BIT(NRF_GPIO_PDN_PIN);
    NRF_FEM_GPIO->OUTSET = BIT(NRF_GPIO_PDN_PIN); // power on the nrf21540

    uint8_t tx_buffer[2] = {0xff, 0xff};
    tx_buffer[0] = 0x80 | 0x14; // read = register 14

    uint8_t data[100];
    uint16_t size = sizeof(data);
    struct spi_buf tx_spi_buf		= {.buf = (void *)&tx_buffer, .len = sizeof(tx_buffer)};
    struct spi_buf_set tx_spi_buf_set 	= {.buffers = &tx_spi_buf, .count = 1};
    struct spi_buf rx_spi_bufs 		= {.buf = data, .len = size};
    struct spi_buf_set rx_spi_buf_set	= {.buffers = &rx_spi_bufs, .count = 1};

    k_sleep(K_MSEC(1000));

    tx_buffer[0] = 0x80 | 0x14; // read = register 14
    err = spi_transceive_dt(&spispec, &tx_spi_buf_set, &rx_spi_buf_set);
    LOG_INF("spi_tranceive_dt() returned %d", err);
    LOG_INF("data returned %02x %02x", data[0],data[1] );

    k_sleep(K_MSEC(1000));

    tx_buffer[0] = 0x80 | 0x15; 
    err = spi_transceive_dt(&spispec, &tx_spi_buf_set, &rx_spi_buf_set);
    LOG_INF("spi_tranceive_dt() returned %d", err);
    LOG_INF("data returned %02x %02x", data[0],data[1] );

    k_sleep(K_MSEC(1000));

    tx_buffer[0] = 0x80 | 0x16; 
    err = spi_transceive_dt(&spispec, &tx_spi_buf_set, &rx_spi_buf_set);
    LOG_INF("spi_tranceive_dt() returned %d", err);
    LOG_INF("data returned %02x %02x", data[0],data[1] );

    k_sleep(K_MSEC(1000));

    tx_buffer[0] = 0x80 | 0x17; 
    err = spi_transceive_dt(&spispec, &tx_spi_buf_set, &rx_spi_buf_set);
    LOG_INF("spi_tranceive_dt() returned %d", err);
    LOG_INF("data returned %02x %02x", data[0],data[1] );

    while (true) {
        err = spi_transceive_dt(&spispec, &tx_spi_buf_set, &rx_spi_buf_set);
        if (err) {
            LOG_ERR("Transceive failed %d", err);
        }
        k_sleep(K_MSEC(1000));
    }
}

int main(void)
{
	LOG_INF("Hello World! %s", CONFIG_BOARD_TARGET);
    
    test_spi();
	return 0;
}
